pub struct Tui {
pub terminal: Terminal<CrosstermBackend<Stdout>>,
pub task: JoinHandle<()>,
pub cancellation_token: CancellationToken,
pub event_rx: UnboundedReceiver<Event>,
pub event_tx: UnboundedSender<Event>,
pub frame_rate: f64,
pub tick_rate: f64,
pub mouse: bool,
pub paste: bool,
}Expand description
Represents the terminal user interface.
This struct encapsulates a ratatui::Terminal and handles the event loop,
mapping crossterm events to weavetui’s own Event enum. It also emits
Tick and Render events at a configurable rate.
Fields
terminal- Theratatuiterminal instance.task- The handle to the Tokio task that runs the event loop.cancellation_token- The cancellation token for the event loop task.event_rx- The receiver for events.event_tx- The sender for events.frame_rate- The frame rate for rendering.tick_rate- The tick rate for application updates.mouse- Flag to enable/disable mouse capture.paste- Flag to enable/disable bracketed paste.
Fields
terminal: Terminal<CrosstermBackend<Stdout>>The ratatui terminal instance.
task: JoinHandle<()>The handle to the Tokio task that runs the event loop.
cancellation_token: CancellationTokenThe cancellation token for the event loop task.
event_rx: UnboundedReceiver<Event>The receiver for events.
event_tx: UnboundedSender<Event>The sender for events.
frame_rate: f64The frame rate for rendering.
tick_rate: f64The tick rate for application updates.
mouse: boolFlag to enable/disable mouse capture.
paste: boolFlag to enable/disable bracketed paste.
Implementations
Sourceimpl Tui
impl Tui
Sourcepub fn new() -> Result<Tui, Error>
pub fn new() -> Result<Tui, Error>
Creates a new Tui instance.
Returns
A Result containing the new Tui instance, or an error if initialization fails.
Sourcepub fn tick_rate(self, tick_rate: f64) -> Tui
pub fn tick_rate(self, tick_rate: f64) -> Tui
Sets the tick rate for the Tui. The tick rate is the number of times per second that the Tui will emit a Event::Tick event. The default tick rate is 4 ticks per second.
The tick is different from the render rate, which is the number of times per second that the application will be drawn to the screen. The tick rate is useful for updating the application state, performing calculations, run background tasks, and other operations that do not require a per-frame operation.
Tick rate will usually be lower than the frame rate.
Sourcepub fn frame_rate(self, frame_rate: f64) -> Tui
pub fn frame_rate(self, frame_rate: f64) -> Tui
Sets the frame rate for the Tui. The frame rate is the number of times per second that the Tui will emit a Event::Render event. The default frame rate is 60 frames per second.
The frame rate is the rate at which the application will be drawn to the screen (by calling
the draw method of each component).
Sourcepub fn mouse(self, mouse: bool) -> Tui
pub fn mouse(self, mouse: bool) -> Tui
Sets whether the Tui should capture mouse events. The default is false.
Sourcepub fn paste(self, paste: bool) -> Tui
pub fn paste(self, paste: bool) -> Tui
Sets whether the Tui should capture paste events. The default is false.
Sourcepub fn enter(&mut self) -> Result<(), Error>
pub fn enter(&mut self) -> Result<(), Error>
Enables cross-term raw mode and enters the alternate screen.
Sourcepub fn exit(&mut self) -> Result<(), Error>
pub fn exit(&mut self) -> Result<(), Error>
Disables cross-term raw mode and exits the alternate screen.
Sourcepub fn suspend(&mut self) -> Result<(), Error>
pub fn suspend(&mut self) -> Result<(), Error>
Suspends the TUI by exiting the alternate screen and disabling raw mode. This is useful for temporarily leaving the application to perform other actions in the terminal.
Methods from Deref<Target = Terminal<CrosstermBackend<Stdout>>>
Sourcepub fn get_frame(&mut self) -> Frame<'_>
pub fn get_frame(&mut self) -> Frame<'_>
Get a Frame object which provides a consistent view into the terminal state for rendering.
Sourcepub fn current_buffer_mut(&mut self) -> &mut Buffer
pub fn current_buffer_mut(&mut self) -> &mut Buffer
Gets the current buffer as a mutable reference.
Sourcepub fn backend_mut(&mut self) -> &mut B
pub fn backend_mut(&mut self) -> &mut B
Gets the backend as a mutable reference
Sourcepub fn flush(&mut self) -> Result<(), Error>
pub fn flush(&mut self) -> Result<(), Error>
Obtains a difference between the previous and the current buffer and passes it to the current backend for drawing.
Sourcepub fn resize(&mut self, area: Rect) -> Result<(), Error>
pub fn resize(&mut self, area: Rect) -> Result<(), Error>
Updates the Terminal so that internal buffers match the requested area.
Requested area will be saved to remain consistent when rendering. This leads to a full clear of the screen.
Sourcepub fn autoresize(&mut self) -> Result<(), Error>
pub fn autoresize(&mut self) -> Result<(), Error>
Queries the backend for size and resizes if it doesn’t match the previous size.
Sourcepub fn draw<F>(
&mut self,
render_callback: F,
) -> Result<CompletedFrame<'_>, Error>
pub fn draw<F>( &mut self, render_callback: F, ) -> Result<CompletedFrame<'_>, Error>
Draws a single frame to the terminal.
Returns a CompletedFrame if successful, otherwise a std::io::Error.
If the render callback passed to this method can fail, use try_draw instead.
Applications should call draw or try_draw in a loop to continuously render the
terminal. These methods are the main entry points for drawing to the terminal.
This method will:
- autoresize the terminal if necessary
- call the render callback, passing it a
Framereference to render to - flush the current internal state by copying the current buffer to the backend
- move the cursor to the last known position if it was set during the rendering closure
- return a
CompletedFramewith the current buffer and the area of the terminal
The CompletedFrame returned by this method can be useful for debugging or testing
purposes, but it is often not used in regular applicationss.
The render callback should fully render the entire frame when called, including areas that are unchanged from the previous frame. This is because each frame is compared to the previous frame to determine what has changed, and only the changes are written to the terminal. If the render callback does not fully render the frame, the terminal will not be in a consistent state.
Examples
use ratatui::{layout::Position, widgets::Paragraph};
// with a closure
terminal.draw(|frame| {
let area = frame.area();
frame.render_widget(Paragraph::new("Hello World!"), area);
frame.set_cursor_position(Position { x: 0, y: 0 });
})?;
// or with a function
terminal.draw(render)?;
fn render(frame: &mut ratatui::Frame) {
frame.render_widget(Paragraph::new("Hello World!"), frame.area());
}Sourcepub fn try_draw<F, E>(
&mut self,
render_callback: F,
) -> Result<CompletedFrame<'_>, Error>
pub fn try_draw<F, E>( &mut self, render_callback: F, ) -> Result<CompletedFrame<'_>, Error>
Tries to draw a single frame to the terminal.
Returns Result::Ok containing a CompletedFrame if successful, otherwise
Result::Err containing the std::io::Error that caused the failure.
This is the equivalent of Terminal::draw but the render callback is a function or
closure that returns a Result instead of nothing.
Applications should call try_draw or draw in a loop to continuously render the
terminal. These methods are the main entry points for drawing to the terminal.
This method will:
- autoresize the terminal if necessary
- call the render callback, passing it a
Framereference to render to - flush the current internal state by copying the current buffer to the backend
- move the cursor to the last known position if it was set during the rendering closure
- return a
CompletedFramewith the current buffer and the area of the terminal
The render callback passed to try_draw can return any Result with an error type that
can be converted into an std::io::Error using the Into trait. This makes it possible
to use the ? operator to propagate errors that occur during rendering. If the render
callback returns an error, the error will be returned from try_draw as an
std::io::Error and the terminal will not be updated.
The CompletedFrame returned by this method can be useful for debugging or testing
purposes, but it is often not used in regular applicationss.
The render callback should fully render the entire frame when called, including areas that are unchanged from the previous frame. This is because each frame is compared to the previous frame to determine what has changed, and only the changes are written to the terminal. If the render function does not fully render the frame, the terminal will not be in a consistent state.
Examples
use std::io;
use ratatui::widgets::Paragraph;
// with a closure
terminal.try_draw(|frame| {
let value: u8 = "not a number".parse().map_err(io::Error::other)?;
let area = frame.area();
frame.render_widget(Paragraph::new("Hello World!"), area);
frame.set_cursor_position(Position { x: 0, y: 0 });
io::Result::Ok(())
})?;
// or with a function
terminal.try_draw(render)?;
fn render(frame: &mut ratatui::Frame) -> io::Result<()> {
let value: u8 = "not a number".parse().map_err(io::Error::other)?;
frame.render_widget(Paragraph::new("Hello World!"), frame.area());
Ok(())
}Sourcepub fn hide_cursor(&mut self) -> Result<(), Error>
pub fn hide_cursor(&mut self) -> Result<(), Error>
Hides the cursor.
Sourcepub fn show_cursor(&mut self) -> Result<(), Error>
pub fn show_cursor(&mut self) -> Result<(), Error>
Shows the cursor.
Sourcepub fn get_cursor(&mut self) -> Result<(u16, u16), Error>
👎Deprecated: the method get_cursor_position indicates more clearly what about the cursor to get
pub fn get_cursor(&mut self) -> Result<(u16, u16), Error>
Gets the current cursor position.
This is the position of the cursor after the last draw call and is returned as a tuple of
(x, y) coordinates.
Sourcepub fn set_cursor(&mut self, x: u16, y: u16) -> Result<(), Error>
👎Deprecated: the method set_cursor_position indicates more clearly what about the cursor to set
pub fn set_cursor(&mut self, x: u16, y: u16) -> Result<(), Error>
Sets the cursor position.
Sourcepub fn get_cursor_position(&mut self) -> Result<Position, Error>
pub fn get_cursor_position(&mut self) -> Result<Position, Error>
Gets the current cursor position.
This is the position of the cursor after the last draw call.
Sourcepub fn set_cursor_position<P>(&mut self, position: P) -> Result<(), Error>
pub fn set_cursor_position<P>(&mut self, position: P) -> Result<(), Error>
Sets the cursor position.
Sourcepub fn clear(&mut self) -> Result<(), Error>
pub fn clear(&mut self) -> Result<(), Error>
Clear the terminal and force a full redraw on the next draw call.
Sourcepub fn swap_buffers(&mut self)
pub fn swap_buffers(&mut self)
Clears the inactive buffer and swaps it with the current buffer
Sourcepub fn insert_before<F>(&mut self, height: u16, draw_fn: F) -> Result<(), Error>
pub fn insert_before<F>(&mut self, height: u16, draw_fn: F) -> Result<(), Error>
Insert some content before the current inline viewport. This has no effect when the viewport is not inline.
The draw_fn closure will be called to draw into a writable Buffer that is height
lines tall. The content of that Buffer will then be inserted before the viewport.
If the viewport isn’t yet at the bottom of the screen, inserted lines will push it towards the bottom. Once the viewport is at the bottom of the screen, inserted lines will scroll the area of the screen above the viewport upwards.
Before:
+---------------------+
| pre-existing line 1 |
| pre-existing line 2 |
+---------------------+
| viewport |
+---------------------+
| |
| |
+---------------------+After inserting 2 lines:
+---------------------+
| pre-existing line 1 |
| pre-existing line 2 |
| inserted line 1 |
| inserted line 2 |
+---------------------+
| viewport |
+---------------------+
+---------------------+After inserting 2 more lines:
+---------------------+
| pre-existing line 2 |
| inserted line 1 |
| inserted line 2 |
| inserted line 3 |
| inserted line 4 |
+---------------------+
| viewport |
+---------------------+If more lines are inserted than there is space on the screen, then the top lines will go directly into the terminal’s scrollback buffer. At the limit, if the viewport takes up the whole screen, all lines will be inserted directly into the scrollback buffer.
Examples
Insert a single line before the current viewport
use ratatui::{
backend::TestBackend,
style::{Color, Style},
text::{Line, Span},
widgets::{Paragraph, Widget},
Terminal,
};
terminal.insert_before(1, |buf| {
Paragraph::new(Line::from(vec![
Span::raw("This line will be added "),
Span::styled("before", Style::default().fg(Color::Blue)),
Span::raw(" the current viewport"),
]))
.render(buf.area, buf);
});Trait Implementations